/************************************************************************/
/*                                                                      */
/*  Licensed Materials - Property of IBM                                */
/*  WA84 SupportPac                                                     */
/*  (c) Copyright IBM Corp. 2001                                        */
/*  US Government Users Restricted Rights - Use, duplication or         */
/*  disclosure restricted by GSA ADP Schedule Contract                  */
/*  with IBM Corp.                                                      */
/*                                                                      */
/************************************************************************/
package com.ibm.workflow.servlet.clientx.wext;

import javax.servlet.http.*;
import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ItemPackage.ExecutionState;
import com.ibm.workflow.servlet.client.*;

/**
* This class enables the end user to save a current executing activity as a draft.
* Once he/she did Draft Save, it can be restart and continue to input again at anytime in the future.
*/
public class RestartAsDraftHandler extends WExtCommandHandler {
    private static final WExtCommand cmd = new WExtCommand("restartAsDraftWorkItem", "draft.gif", "Command.RestartAsDraftWorkItem");

    /**
    * Returns information about the handler.
    * @return   a String containing handler information
    */
    public String getHandlerInfo() {
        return "MQWF WebClient Extentions V1.0 - RestartAsDraftHandler";
    }

    /**
    * Initializes the handler.
    * @param config     the servlet's configuration and initialization parameters
    */
    public void init(Config cfg) {
        super.init(cfg);
        getContext().log("init RestartAsDraftHandler", false);
    }

    /**
    * Returns a String that contains HTML code to trigger the restart as draft command.
    * @param list    denotes the list for which the trigger tag is queried.
    * @param context the RequestContext which was passed to the Viewer
    *                calling this method.
    * @return       a string containing HTML tags for the icons to be displayed on list.
    */
    public String getTriggerTagFor(int list, RequestContext context) {
        if (list != SessionContext.WORKLIST) {
            context.log(" List is not Work List.", false);
            return "";
        }

        try {
            WorkItem workitem = context.getWorkItem();
            if (workitem == null) {
                context.log(" WorkItem is null.", false);
                return "";
            }

            if (workitem.state().value() != ExecutionState.EXECUTED.value())
                return "";

            ProcessInstance process = workitem.processInstance();
            if (process.state().value() != ExecutionState.RUNNING.value())
                return "";

            String oid = workitem.persistentOid();
            String name = workitem.name();
            String tag = cmd.getTriggerTag(context, oid, name);

            return cmd.getTriggerTag(context, oid, name);
        }
        catch (FmcException ex) {
            context.log(" FmcException:" + ex.toString(), false);
            return "";
        }
    }


    /**
    * This custom command first retrieves the output container data of the work item
    * whose OID matches the 'id' parameter in the HTTP-request.
    * Then the command creates a ReuseData object as a request bean to keep old data for reuse.
    * The resulting HTML data is created by checkOutWorkItem rendering.
    * @param request        the request the client has made of the servlet
    * @param response       the response the servlet sends to the client
    * @return               the result page to be sent to the client
    */
    public ResponsePage restartAsDraftWorkItem(HttpServletRequest request,
                                            HttpServletResponse response)
                                        throws ClientException {
        getContext().log("restartAsDraftWorkItem called.", false);

        try {
            SessionContext sc = getSessionContext(request);
            BuiltinHandler bh = sc.getBuiltinHandler();
            RequestContext context = new RequestContext(request);
            String id = request.getParameter("id");
            WorkItem wi = sc.getWorkItem(sc.getWorkListOid(), id);

            if (wi == null) {
                getContext().log("  Workitem " + id + " not found.", false);
                throw new ClientException(request, context.get("Error.WorkItemNotFound", id));
            }

            ReadWriteContainer container = wi.outContainer();
            ReuseData data = new ReuseData(container, context);
            data.setDraftSaved(true);
            request.setAttribute(getDataBeanId(), data);

            wi.restart();
            context.setWorkItem(wi);

            context = bh.checkOutWorkItem(request, response);
            ResponsePage page = bh.render("checkOutWorkItem", context);
            getContext().log("restartAsDraftWorkItem done.", false);
            return page;
        }
        catch (FmcException ex) {
            throw new ClientException(request, ex);
        }
    }

    /**
    * This custom command check-in work item.
    * @param request        the request the client has made of the servlet
    * @param response       the response the servlet sends to the client
    * @return               the result page to be sent to the client
    */
    public ResponsePage completeWorkItem(HttpServletRequest request,
                                HttpServletResponse response) throws ClientException {
        return checkInOrCompleteWorkItem(request, response, true);
    }

    /**
    * This custom command does not cancelWorkItem but check-in with old data.
    * @param request        the request the client has made of the servlet
    * @param response       the response the servlet sends to the client
    * @return               the result page to be sent to the client
    */
    public ResponsePage retrieveDraftWorkItem(HttpServletRequest request,
                                HttpServletResponse response) throws ClientException {
        return cancelOrRetrieveDraftWorkItem(request, response, true);
    }
}


